//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "VampLpt.h"

//---------------------------------------------------------------------------

// 
CVampLpt::CVampLpt(VOID)
{
 hdrv = INVALID_HANDLE_VALUE;
 Ready();
};

// 
CVampLpt::~CVampLpt(VOID)
{
 CloseHandle(hdrv);
};

//  .  TRUE,   .
BOOL CVampLpt::Ready(VOID)
{
 if (hdrv==INVALID_HANDLE_VALUE)
 {
  SetLastError(NO_ERROR);
  hdrv = CreateFile(VAMP_DRIVER_NAME, //   
                    GENERIC_READ|GENERIC_WRITE,
                    FILE_SHARE_READ|FILE_SHARE_WRITE,
                    NULL,
                    OPEN_EXISTING,
                    FILE_ATTRIBUTE_NORMAL,
                    NULL);
  //                    
  ULONG signature = 0, cb;
  DeviceIoControl(hdrv,
                  IOCTL_VAMP_CHECK,
                  &signature, sizeof(ULONG),
                  &signature, sizeof(ULONG),
                  &cb,NULL);
  if (VAMPIRE_IOCTL_SIGNATURE!=signature)
  { //  !
   CloseHandle(hdrv);
   hdrv=INVALID_HANDLE_VALUE;
  } else
  {
   ULONG index = 0;
   while (index<LPT2)
   {
    //   event-,   "" 
    UCHAR evname[64];
    wsprintf(evname,"LPTVAMPIREV0P%u",index);
    //     
    ev[index++] = OpenEvent(SYNCHRONIZE,FALSE,evname);
   };
  };
 };
 return (hdrv!=INVALID_HANDLE_VALUE);
};

//      
BOOL CVampLpt::GetPortMode(ULONG PortIndex, PPORT_DATA pPortModeData)
{
 if (!Ready())       { return FALSE; };
 if (!pPortModeData) { return FALSE; };

 PORT_CONTROL_REQUEST req; ZeroMemory(&req,sizeof(req)); //  
 req.PortIndex   = PortIndex-LPT1;
 req.RequestCode = VAMP_GET_PORT_DATA;

 SetLastError(ERROR_SUCCESS);
 ULONG cb = 0;
 return DeviceIoControl(hdrv, //      .
                        IOCTL_VAMP_CONTROL_REQUEST,
                        &req, sizeof(req),
                        pPortModeData, sizeof(PORT_DATA),
                        &cb,NULL);
};

//  
BOOL CVampLpt::LockPort(ULONG PortIndex)
{
 if (!Ready())       { return FALSE; };

 PORT_CONTROL_REQUEST req; ZeroMemory(&req,sizeof(req)); //  
 req.PortIndex   = PortIndex-LPT1;
 req.RequestCode = VAMP_LOCK_PORT;

 SetLastError(ERROR_SUCCESS);
 ULONG cb = 0;
 return DeviceIoControl(hdrv, //      .
                        IOCTL_VAMP_CONTROL_REQUEST,
                        &req, sizeof(req),
                        NULL, 0,
                        &cb,NULL);
};

//  
BOOL CVampLpt::UnlockPort(ULONG PortIndex)
{
 if (!Ready())       { return FALSE; };

 PORT_CONTROL_REQUEST req; ZeroMemory(&req,sizeof(req)); //  
 req.PortIndex   = PortIndex-LPT1;
 req.RequestCode = VAMP_UNLOCK_PORT;

 SetLastError(ERROR_SUCCESS);
 ULONG cb = 0;
 return DeviceIoControl(hdrv, //      .
                        IOCTL_VAMP_CONTROL_REQUEST,
                        &req, sizeof(req),
                        NULL, 0,
                        &cb,NULL);
};

// /  
BOOL CVampLpt::ReadWrite(ULONG PortIndex, //  
                         PADS  Source,    //     ADS,    
                         PADS  Dest,      //     ADS,     .
                         ULONG PairCount) //   ADS   Source
{
 SetLastError(ERROR_SUCCESS);
 if (!Ready()){ return FALSE; }; // ...
 if (!Source) { return TRUE;  }; //  ?
 if ((PairCount>RW_REQ_MAX_ADS_COUNT)||(PairCount==0))
 { // -   !
  SetLastError(ERROR_INVALID_PARAMETER);
  return FALSE;
 };
 //   
 ULONG reqlen = sizeof(READ_WRITE_REQUEST)+PairCount*sizeof(ADS)+1;       //   
 PREAD_WRITE_REQUEST preq = (PREAD_WRITE_REQUEST)LocalAlloc(LPTR,reqlen);
 if (!preq) { return ERROR_INSUFFICIENT_BUFFER; }; //  ?
 PADS pads = &preq->Ads[0];   //     ADS  
 //    
 preq->PortIndex = PortIndex-LPT1; //  
 preq->PairCount = PairCount; //   ADS (-)
 CopyMemory(pads,Source,PairCount*sizeof(ADS)); // 
 //  
 ULONG cb = 0;
 BOOL res = DeviceIoControl(hdrv,
                            IOCTL_VAMP_READ_WRITE_DATA,
                            preq, reqlen,
                            preq, reqlen,
                            &cb,NULL);
 res = res&(cb==reqlen); //  ,     :)
 if (res)
 { // !   .  .   .
  if (Dest)
  {
   CopyMemory(Dest,pads,PairCount*sizeof(ADS));
  };
 };

 LocalFree(preq); //  
 return res;
};

//   .  ,   .
UCHAR CVampLpt::ReadRegister(ULONG PortIndex, //  
                             ULONG RegOffset, //  
                             BOOL  *result)   // TRUE,    .  ==NULL,   .
{
 ADS ads = { ADS_READ|(ADS_OFFSET_MASK&RegOffset), 0xFF }; //  
 BOOL res = ReadWrite(PortIndex,&ads,&ads,1); //  -
 if (result) { *result = res; };         //  
 return ads.Data;
};

//    
BOOL CVampLpt::WriteRegister(ULONG PortIndex, //  
                             ULONG RegOffset, //  
                             UCHAR Data)      //     
{
 ADS ads = { ADS_OFFSET_MASK&RegOffset, Data }; //  
 return ReadWrite(PortIndex,&ads,NULL,1);     //  -
};

//     RAW-   
HANDLE CVampLpt::OpenParPort(ULONG PortIndex)
{
 SetLastError(ERROR_SUCCESS);
 if (PortIndex>LPT2)
 { //   ?
  SetLastError(ERROR_INVALID_PARAMETER);
  return INVALID_HANDLE_VALUE;
 };
 //  
 UnlockPort(PortIndex);
 //   
 UCHAR LptName[32];
 wsprintf(LptName,"\\\\.\\LPT%u",PortIndex);
 return CreateFile(LptName,
                   GENERIC_READ|GENERIC_WRITE,
                   FILE_SHARE_READ|FILE_SHARE_WRITE,
                   NULL,
                   OPEN_EXISTING,
                   FILE_ATTRIBUTE_NORMAL,
                   NULL);
};


// /  .
BOOL CVampLpt::EnableInterrupt(ULONG PortIndex, //  
                               BOOL Enable)     // TRUE -  
{
 SetLastError(ERROR_SUCCESS);
 if (!Ready()){ return FALSE; }; // ...

 BOOL intenabled = FALSE;
 PORT_CONTROL_REQUEST req; ZeroMemory(&req,sizeof(req)); //  
 req.PortIndex   = PortIndex-LPT1;
 req.RequestCode = Enable?VAMP_USE_INTERRUPT:VAMP_FREE_INTERRUPT;

 SetLastError(ERROR_SUCCESS);
 ULONG cb = 0;
 if (!DeviceIoControl(hdrv, //      .
                      IOCTL_VAMP_CONTROL_REQUEST,
                      &req, sizeof(req),
                      &intenabled, sizeof(intenabled),
                      &cb,NULL))
 {
  return FALSE;
 } else
 {
  return (intenabled==Enable)&&InterruptChReady(PortIndex);
 };
};

//  
BOOL CVampLpt::WaitForInterrupt(ULONG PortIndex, //  
                                ULONG MaxTime)   //   ,  .
{
 SetLastError(ERROR_SUCCESS);
 if (!Ready()) { return FALSE; }; // ...
 if (PortIndex>LPT2) { return FALSE; }; // -!

 BOOL hasinterrupt = FALSE; //   
 if (ev[PortIndex-LPT1])
 { // !          (max speed)
  hasinterrupt = (WAIT_OBJECT_0==WaitForSingleObject(ev[PortIndex-LPT1],MaxTime)); //  -->  .
 } else
 { //  ...    -    .
  ULONG Time = 0;
  PORT_CONTROL_REQUEST req; ZeroMemory(&req,sizeof(req));
  while ((Time<MaxTime)&&(!hasinterrupt))
  {
   req.PortIndex   = PortIndex-LPT1;
   req.RequestCode = VAMP_POLL_INTERRUPT;
   ULONG cb = 0;
   DeviceIoControl(hdrv, //      .
                   IOCTL_VAMP_CONTROL_REQUEST,
                   &req, sizeof(req),
                   &hasinterrupt, sizeof(hasinterrupt),
                   &cb,NULL);
   if (!hasinterrupt) { Sleep(1); ++Time; }; //  
  };
 };

 return hasinterrupt;
};


//      
BOOL CVampLpt::InterruptChReady(ULONG PortIndex)
{
 if (!Ready()) { return FALSE; };
 if (PortIndex>LPT2) { return FALSE; };
 return (0!=ev[PortIndex-LPT1]);
};

//---------------------------------------------------------------------------

#pragma package(smart_init)
